@@ -66,6 +66,7 @@
 #include "uri.h"
 #include "FileEntry.h"
 #include "error_code.h"
+#include "SocketRecvBuffer.h"
 #ifdef ENABLE_ASYNC_DNS
 #include "AsyncNameResolver.h"
 #endif // ENABLE_ASYNC_DNS
@@ -75,20 +76,23 @@
 
 namespace aria2 {
 
-AbstractCommand::AbstractCommand(cuid_t cuid,
-                                 const SharedHandle<Request>& req,
-                                 const SharedHandle<FileEntry>& fileEntry,
-                                 RequestGroup* requestGroup,
-                                 DownloadEngine* e,
-                                 const SocketHandle& s,
-                                 bool incNumConnection):
-  Command(cuid), checkPoint_(global::wallclock),
-  timeout_(requestGroup->getTimeout()),
-  requestGroup_(requestGroup),
-  req_(req), fileEntry_(fileEntry), e_(e), socket_(s),
-  checkSocketIsReadable_(false), checkSocketIsWritable_(false),
-  nameResolverCheck_(false),
-  incNumConnection_(incNumConnection)
+AbstractCommand::AbstractCommand
+(cuid_t cuid,
+ const SharedHandle<Request>& req,
+ const SharedHandle<FileEntry>& fileEntry,
+ RequestGroup* requestGroup,
+ DownloadEngine* e,
+ const SocketHandle& s,
+ const SharedHandle<SocketRecvBuffer>& socketRecvBuffer,
+ bool incNumConnection)
+  : Command(cuid), checkPoint_(global::wallclock),
+    timeout_(requestGroup->getTimeout()),
+    requestGroup_(requestGroup),
+    req_(req), fileEntry_(fileEntry), e_(e), socket_(s),
+    socketRecvBuffer_(socketRecvBuffer),
+    checkSocketIsReadable_(false), checkSocketIsWritable_(false),
+    nameResolverCheck_(false),
+    incNumConnection_(incNumConnection)
 {
   if(socket_ && socket_->isOpen()) {
     setReadCheckSocket(socket_);
@@ -169,7 +173,9 @@ bool AbstractCommand::execute() {
         }
       }
     }
-    if((checkSocketIsReadable_ && readEventEnabled()) ||
+    if((checkSocketIsReadable_ &&
+        (readEventEnabled() ||
+         (socketRecvBuffer_ && !socketRecvBuffer_->bufferEmpty()))) ||
        (checkSocketIsWritable_ && writeEventEnabled()) ||
        hupEventEnabled() ||
 #ifdef ENABLE_ASYNC_DNS
@@ -234,13 +240,17 @@ bool AbstractCommand::execute() {
           e_->getRequestGroupMan()->getOrCreateServerStat(req_->getHost(),
                                                           req_->getProtocol());
         ss->setError();
-        // Purging IP address cache to renew IP address.
-        A2_LOG_DEBUG(fmt("CUID#%lld - Marking IP address %s as bad",
-                         getCuid(),
-                         req_->getConnectedAddr().c_str()));
-        e_->markBadIPAddress(req_->getConnectedHostname(),
-                             req_->getConnectedAddr(),
-                             req_->getConnectedPort());
+        // When DNS query was timeout, req_->getConnectedAddr() is
+        // empty.
+        if(!req_->getConnectedAddr().empty()) {
+          // Purging IP address cache to renew IP address.
+          A2_LOG_DEBUG(fmt("CUID#%lld - Marking IP address %s as bad",
+                           getCuid(),
+                           req_->getConnectedAddr().c_str()));
+          e_->markBadIPAddress(req_->getConnectedHostname(),
+                               req_->getConnectedAddr(),
+                               req_->getConnectedPort());
+        }
         if(e_->findCachedIPAddress
            (req_->getConnectedHostname(), req_->getConnectedPort()).empty()) {
           A2_LOG_DEBUG(fmt("CUID#%lld - All IP addresses were marked bad."
@@ -300,6 +310,9 @@ bool AbstractCommand::execute() {
       tryReserved();
       return true;
     } else {
+      Timer wakeTime(global::wallclock);
+      wakeTime.advance(getOption()->getAsInt(PREF_RETRY_WAIT));
+      req_->setWakeTime(wakeTime);
       return prepareForRetry(0);
     }
   } catch(DownloadFailureException& err) {
@@ -341,6 +354,12 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
     getSegmentMan()->cancelSegment(getCuid());
   }
   if(req_) {
+    // Reset persistentConnection and maxPipelinedRequest to handle
+    // the situation where remote server returns Connection: close
+    // after several pipelined requests.
+    req_->supportsPersistentConnection(true);
+    req_->setMaxPipelinedRequest(1);
+
     fileEntry_->poolRequest(req_);
     A2_LOG_DEBUG(fmt("CUID#%lld - Pooling request URI=%s",
                      getCuid(), req_->getUri().c_str()));
@@ -631,7 +650,13 @@ void AbstractCommand::initAsyncNameResolver(const std::string& hostname)
   } else {
     family = AF_INET;
   }
-  asyncNameResolver_.reset(new AsyncNameResolver(family));
+  asyncNameResolver_.reset
+    (new AsyncNameResolver(family
+#ifdef HAVE_ARES_ADDR_NODE
+                           ,
+                           e_->getAsyncDNSServers()
+#endif // HAVE_ARES_ADDR_NODE
+                           ));
   A2_LOG_INFO(fmt(MSG_RESOLVING_HOSTNAME,
                   getCuid(),
                   hostname.c_str()));
@@ -846,4 +871,12 @@ const SharedHandle<PieceStorage>& AbstractCommand::getPieceStorage() const
   return requestGroup_->getPieceStorage();
 }
 
+void AbstractCommand::checkSocketRecvBuffer()
+{
+  if(!socketRecvBuffer_->bufferEmpty()) {
+    setStatus(Command::STATUS_ONESHOT_REALTIME);
+    e_->setNoWait(true);
+  }
+}
+
 } // namespace aria2
