@@ -58,6 +58,15 @@
 #include "DHTPeerAnnounceStorage.h"
 #include "DHTSetup.h"
 #include "DHTRegistry.h"
+#include "DHTNode.h"
+#include "DHTRoutingTable.h"
+#include "DHTTaskQueue.h"
+#include "DHTTaskFactory.h"
+#include "DHTTokenTracker.h"
+#include "DHTMessageDispatcher.h"
+#include "DHTMessageReceiver.h"
+#include "DHTMessageFactory.h"
+#include "DHTMessageCallback.h"
 #include "BtProgressInfoFile.h"
 #include "BtAnnounce.h"
 #include "BtRuntime.h"
@@ -69,15 +78,16 @@
 #include "LpdMessageDispatcher.h"
 #include "message.h"
 #include "SocketCore.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
 #include "DlAbortEx.h"
+#include "array_fun.h"
+#include "DownloadContext.h"
+#include "PieceStorage.h"
+#include "PeerStorage.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-BtSetup::BtSetup():logger_(LogFactory::getInstance()) {}
+BtSetup::BtSetup() {}
 
 void BtSetup::setup(std::vector<Command*>& commands,
                     RequestGroup* requestGroup,
@@ -126,15 +136,25 @@ void BtSetup::setup(std::vector<Command*>& commands,
     commands.push_back(c);
   }
 
-  if((metadataGetMode || !torrentAttrs->privateTorrent) &&
-     DHTSetup::initialized()) {
-    DHTGetPeersCommand* command =
-      new DHTGetPeersCommand(e->newCUID(), requestGroup, e);
-    command->setTaskQueue(DHTRegistry::getData().taskQueue);
-    command->setTaskFactory(DHTRegistry::getData().taskFactory);
-    command->setBtRuntime(btRuntime);
-    command->setPeerStorage(peerStorage);
-    commands.push_back(command);
+  if(metadataGetMode || !torrentAttrs->privateTorrent) {
+    if(DHTRegistry::isInitialized()) {
+      DHTGetPeersCommand* command =
+        new DHTGetPeersCommand(e->newCUID(), requestGroup, e);
+      command->setTaskQueue(DHTRegistry::getData().taskQueue);
+      command->setTaskFactory(DHTRegistry::getData().taskFactory);
+      command->setBtRuntime(btRuntime);
+      command->setPeerStorage(peerStorage);
+      commands.push_back(command);
+    }
+    if(DHTRegistry::isInitialized6()) {
+      DHTGetPeersCommand* command =
+        new DHTGetPeersCommand(e->newCUID(), requestGroup, e);
+      command->setTaskQueue(DHTRegistry::getData6().taskQueue);
+      command->setTaskFactory(DHTRegistry::getData6().taskFactory);
+      command->setBtRuntime(btRuntime);
+      command->setPeerStorage(peerStorage);
+      commands.push_back(command);
+    }
   }
   if(!metadataGetMode) {
     SharedHandle<UnionSeedCriteria> unionCri(new UnionSeedCriteria());
@@ -164,25 +184,45 @@ void BtSetup::setup(std::vector<Command*>& commands,
     }
   }
   if(PeerListenCommand::getNumInstance() == 0) {
-    PeerListenCommand* listenCommand = PeerListenCommand::getInstance(e);
-    IntSequence seq =util::parseIntRange(e->getOption()->get(PREF_LISTEN_PORT));
-    uint16_t port;
-    if(listenCommand->bindPort(port, seq)) {
-      btRuntime->setListenPort(port);
-      // Add command to DownloadEngine directly.
-      e->addCommand(listenCommand);
-    } else {
-      delete listenCommand;
+    static int families[] = { AF_INET, AF_INET6 };
+    size_t familiesLength = e->getOption()->getAsBool(PREF_DISABLE_IPV6)?1:2;
+    for(size_t i = 0; i < familiesLength; ++i) {
+      PeerListenCommand* listenCommand =
+        PeerListenCommand::getInstance(e, families[i]);
+      bool ret;
+      uint16_t port;
+      if(btRuntime->getListenPort()) {
+        IntSequence seq =
+          util::parseIntRange(util::uitos(btRuntime->getListenPort()));
+        ret = listenCommand->bindPort(port, seq);
+      } else {
+        IntSequence seq =
+          util::parseIntRange(e->getOption()->get(PREF_LISTEN_PORT));
+        ret = listenCommand->bindPort(port, seq);
+      }
+      if(ret) {
+        btRuntime->setListenPort(port);
+        // Add command to DownloadEngine directly.
+        e->addCommand(listenCommand);
+      } else {
+        delete listenCommand;
+      }
+    }
+    if(PeerListenCommand::getNumInstance() == 0) {
       throw DL_ABORT_EX(_("Errors occurred while binding port.\n"));
     }
   } else {
-    PeerListenCommand* listenCommand = PeerListenCommand::getInstance(e);
+    PeerListenCommand* listenCommand =
+      PeerListenCommand::getInstance(e, AF_INET);
+    if(!listenCommand) {
+      listenCommand = PeerListenCommand::getInstance(e, AF_INET6);
+    }
     btRuntime->setListenPort(listenCommand->getPort());
   }
   if(option->getAsBool(PREF_BT_ENABLE_LPD) &&
      (metadataGetMode || !torrentAttrs->privateTorrent)) {
     if(LpdReceiveMessageCommand::getNumInstance() == 0) {
-      logger_->info("Initializing LpdMessageReceiver.");
+      A2_LOG_INFO("Initializing LpdMessageReceiver.");
       SharedHandle<LpdMessageReceiver> receiver
         (new LpdMessageReceiver(LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT));
       bool initialized = false;
@@ -206,15 +246,15 @@ void BtSetup::setup(std::vector<Command*>& commands,
         }
       }
       if(initialized) {
-        logger_->info("LpdMessageReceiver initialized. multicastAddr=%s:%u,"
-                      " localAddr=%s",
-                      LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT,
-                      receiver->getLocalAddress().c_str());
+        A2_LOG_INFO(fmt("LpdMessageReceiver initialized. multicastAddr=%s:%u,"
+                        " localAddr=%s",
+                        LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT,
+                        receiver->getLocalAddress().c_str()));
         LpdReceiveMessageCommand* cmd =
           LpdReceiveMessageCommand::getInstance(e, receiver);
         e->addCommand(cmd);
       } else {
-        logger_->info("LpdMessageReceiver not initialized.");
+        A2_LOG_INFO("LpdMessageReceiver not initialized.");
       }
     }
     if(LpdReceiveMessageCommand::getNumInstance()) {
@@ -222,20 +262,20 @@ void BtSetup::setup(std::vector<Command*>& commands,
         bittorrent::getInfoHash(requestGroup->getDownloadContext());
       SharedHandle<LpdMessageReceiver> receiver =
         LpdReceiveMessageCommand::getInstance()->getLpdMessageReceiver();
-      logger_->info("Initializing LpdMessageDispatcher.");      
+      A2_LOG_INFO("Initializing LpdMessageDispatcher.");      
       SharedHandle<LpdMessageDispatcher> dispatcher
         (new LpdMessageDispatcher
          (std::string(&infoHash[0], &infoHash[INFO_HASH_LENGTH]),
           btRuntime->getListenPort(),
           LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT));
       if(dispatcher->init(receiver->getLocalAddress(), /*ttl*/1, /*loop*/0)) {
-        logger_->info("LpdMessageDispatcher initialized.");      
+        A2_LOG_INFO("LpdMessageDispatcher initialized.");      
         LpdDispatchMessageCommand* cmd =
           new LpdDispatchMessageCommand(e->newCUID(), dispatcher, e);
         cmd->setBtRuntime(btRuntime);
         e->addCommand(cmd);
       } else {
-        logger_->info("LpdMessageDispatcher not initialized.");
+        A2_LOG_INFO("LpdMessageDispatcher not initialized.");
       }
     }
   }
