@@ -10,7 +10,6 @@
 #include "DHTMessageTrackerEntry.h"
 #include "DHTRoutingTable.h"
 #include "MockDHTMessageFactory.h"
-#include "bencode.h"
 
 namespace aria2 {
 
@@ -28,18 +27,6 @@ public:
   void testMessageArrived();
 
   void testHandleTimeout();
-
-  class MockDHTMessageCallback2:public MockDHTMessageCallback {
-  public:
-    uint32_t _countOnRecivedCalled;
-
-    MockDHTMessageCallback2():_countOnRecivedCalled(0) {}
-
-    virtual void onReceived(const SharedHandle<DHTMessage>& message)
-    {
-      ++_countOnRecivedCalled;
-    }
-  };
 };
 
 
@@ -66,51 +53,48 @@ void DHTMessageTrackerTest::testMessageArrived()
   m3->getRemoteNode()->setIPAddress("192.168.0.3");
   m3->getRemoteNode()->setPort(6883);
 
-  SharedHandle<MockDHTMessageCallback2> c2(new MockDHTMessageCallback2());
-  
   DHTMessageTracker tracker;
   tracker.setRoutingTable(routingTable);
   tracker.setMessageFactory(factory);
-  tracker.addMessage(m1);
-  tracker.addMessage(m2, c2);
-  tracker.addMessage(m3);
+  tracker.addMessage(m1, DHT_MESSAGE_TIMEOUT);
+  tracker.addMessage(m2, DHT_MESSAGE_TIMEOUT);
+  tracker.addMessage(m3, DHT_MESSAGE_TIMEOUT);
 
   {
-    BDE resDict = BDE::dict();
-    resDict["t"] = m2->getTransactionID();
+    Dict resDict;
+    resDict.put("t", m2->getTransactionID());
     
     std::pair<SharedHandle<DHTMessage>, SharedHandle<DHTMessageCallback> > p =
-      tracker.messageArrived(resDict, m2->getRemoteNode()->getIPAddress(),
+      tracker.messageArrived(&resDict, m2->getRemoteNode()->getIPAddress(),
                              m2->getRemoteNode()->getPort());
     SharedHandle<DHTMessage> reply = p.first;
 
-    CPPUNIT_ASSERT(!reply.isNull());
-    CPPUNIT_ASSERT_EQUAL((uint32_t)0, c2->_countOnRecivedCalled);
-    CPPUNIT_ASSERT(tracker.getEntryFor(m2).isNull());
+    CPPUNIT_ASSERT(reply);
+    CPPUNIT_ASSERT(!tracker.getEntryFor(m2));
     CPPUNIT_ASSERT_EQUAL((size_t)2, tracker.countEntry());
   }
   {
-    BDE resDict = BDE::dict();
-    resDict["t"] = m3->getTransactionID();
+    Dict resDict;
+    resDict.put("t", m3->getTransactionID());
 
     std::pair<SharedHandle<DHTMessage>, SharedHandle<DHTMessageCallback> > p =
-      tracker.messageArrived(resDict, m3->getRemoteNode()->getIPAddress(),
+      tracker.messageArrived(&resDict, m3->getRemoteNode()->getIPAddress(),
                              m3->getRemoteNode()->getPort());
     SharedHandle<DHTMessage> reply = p.first;
 
-    CPPUNIT_ASSERT(!reply.isNull());
-    CPPUNIT_ASSERT(tracker.getEntryFor(m3).isNull());
+    CPPUNIT_ASSERT(reply);
+    CPPUNIT_ASSERT(!tracker.getEntryFor(m3));
     CPPUNIT_ASSERT_EQUAL((size_t)1, tracker.countEntry());
   }
   {
-    BDE resDict = BDE::dict();
-    resDict["t"] = m1->getTransactionID();
+    Dict resDict;
+    resDict.put("t", m1->getTransactionID());
 
     std::pair<SharedHandle<DHTMessage>, SharedHandle<DHTMessageCallback> > p =
-      tracker.messageArrived(resDict, "192.168.1.100", 6889);
+      tracker.messageArrived(&resDict, "192.168.1.100", 6889);
     SharedHandle<DHTMessage> reply = p.first;
 
-    CPPUNIT_ASSERT(reply.isNull());
+    CPPUNIT_ASSERT(!reply);
   }
 }
 
